@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_plugin.h,v 1.9 2004/09/25 15:32:01 pollux Exp $
+/*      $Id: xfwm4_plugin.h,v 1.11 2004/10/12 06:50:27 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -23,8 +23,10 @@
 
 #define RCDIR    "mcs_settings"
 #define OLDRCDIR "settings"
-#define RCFILE   "xfwm4.xml"
-#define CHANNEL  "xfwm4"
+#define RCFILE1   "xfwm4.xml"
+#define RCFILE2   "xfwm4_keys.xml"
+#define CHANNEL1  "xfwm4"
+#define CHANNEL2  "xfwm4_keys"
 #define PLUGIN_NAME "xfwm4"
 
 #define DEFAULT_THEME "Default"
@@ -64,10 +66,7 @@ struct _ThemeInfo
 typedef struct _shortcut_tree_foreach_struct shortcut_tree_foreach_struct;
 struct _shortcut_tree_foreach_struct
 {
-    gchar *key;
-    gboolean modifier_control;
-    gboolean modifier_shift;
-    gboolean modifier_alt;
+    gchar *shortcut;
     gchar *path;
     gboolean found;
 };
