@@ -1,4 +1,4 @@
-/*      $Id: client.h,v 1.122 2004/09/09 22:54:17 olivier Exp $
+/*      $Id: client.h,v 1.127 2004/10/26 20:42:22 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -35,12 +35,6 @@
 #include <X11/cursorfont.h>
 #include <X11/extensions/shape.h>
 
-#ifdef HAVE_COMPOSITOR
-#include <X11/extensions/Xcomposite.h>
-#include <X11/extensions/Xdamage.h>
-#include <X11/extensions/Xrender.h>
-#endif /* HAVE_COMPOSITOR */
-
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <libxfcegui4/libxfcegui4.h>
@@ -149,9 +143,10 @@
                                        !(c->type & (WINDOW_DESKTOP | WINDOW_DOCK)) && \
                                        !FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN))
 
-#define WINDOW_TYPE_DIALOG               (WINDOW_DIALOG | WINDOW_MODAL_DIALOG)
-#define WINDOW_TYPE_DONT_PLACE           (WINDOW_DESKTOP | WINDOW_DOCK | WINDOW_SPLASHSCREEN)
-    
+#define WINDOW_TYPE_DIALOG             (WINDOW_DIALOG | WINDOW_MODAL_DIALOG)
+#define WINDOW_TYPE_DONT_PLACE         (WINDOW_DESKTOP | WINDOW_DOCK | WINDOW_SPLASHSCREEN)
+#define WINDOW_TYPE_DONT_FOCUS         (WINDOW_DOCK)
+
 
 /* Convenient macros */
 #define FLAG_TEST(flag,bits)                   (flag & (bits))
@@ -253,28 +248,13 @@ struct _Client
     guint blink_timeout_id;
     /* Pixmap caching */
     ClientPixmapCache pm_cache;
+    /* Opacity for the compositor */
+    guint opacity;
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     /* Startup notification */
     char *startup_id;
 #endif /* HAVE_LIBSTARTUP_NOTIFICATION */
-
-#ifdef HAVE_COMPOSITOR
-    Damage damage;
-    XserverRegion last_painted_extents;
-  
-    Picture picture;
-    XserverRegion border_size;
-
-#if HAVE_NAME_WINDOW_PIXMAP
-    Pixmap name_window_pixmap;
-#endif /* HAVE_NAME_WINDOW_PIXMAP */
-  
-  unsigned int managed : 1;
-  unsigned int damaged : 1;
-  unsigned int viewable : 1;
-
-#endif /* HAVE_COMPOSITOR */
 };
 
 extern Client *clients;
