@@ -1,4 +1,4 @@
-/*      $Id: compositor.h,v 1.6 2004/10/02 11:45:21 olivier Exp $
+/*      $Id: compositor.h,v 1.12 2004/10/20 20:30:27 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -32,17 +32,22 @@
 #include "screen.h"
 #include "client.h"
 
-void compositorWindowMap (DisplayInfo *, Window);
-void compositorWindowUnmap (DisplayInfo *, Window);
-void compositorAddWindow (DisplayInfo *, Window, Client *c);
-void compositorRemoveWindow (DisplayInfo *, Window);
+void     compositorMapWindow        (DisplayInfo *, Window);
+void     compositorUnmapWindow      (DisplayInfo *, Window);
+void     compositorAddWindow        (DisplayInfo *, Window, Client *c);
+void     compositorRemoveWindow     (DisplayInfo *, Window);
 
-void compositorHandleEvent (DisplayInfo *, XEvent *);
-void compositorInitDisplay (DisplayInfo *);
+void     compositorHandleEvent      (DisplayInfo *, XEvent *);
+gboolean compositorCheckDamageEvent (DisplayInfo *display_info);
+void     compositorInitDisplay      (DisplayInfo *);
+void     compositorRepairDisplay    (DisplayInfo *);
 
-void compositorManageScreen (ScreenInfo *);
-void compositorUnmanageScreen (ScreenInfo *);
+gboolean compositorManageScreen     (ScreenInfo *, gboolean);
+void     compositorUnmanageScreen   (ScreenInfo *);
+void     compositorRepairScreen     (DisplayInfo *);
 
-void compositorDamageWindow (DisplayInfo *, Window);
+void     compositorWindowSetOpacity (DisplayInfo *, Window, guint);
+void     compositorDamageWindow     (DisplayInfo *, Window);
+void     compositorUpdateWindow     (DisplayInfo *, Window, gint, gint, gboolean);
 
 #endif /* INC_COMPOSITOR_H */
