@@ -1,4 +1,4 @@
-/*      $Id: display.c,v 1.9 2004/09/26 12:27:56 olivier Exp $
+/*      $Id: display.c,v 1.11 2004/10/24 13:32:37 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -64,6 +64,10 @@ myDisplayInit (GdkDisplay *gdisplay)
 
     display->shape = 
         XShapeQueryExtension (display->dpy, &display->shape_event, &dummy);
+    if (!display->shape)
+    {
+        g_warning ("%s: The display does not support the XShape extension.", PACKAGE);
+    }
 
     display->root_cursor = 
         XCreateFontCursor (display->dpy, XC_left_ptr);
@@ -383,4 +387,3 @@ myDisplayGetScreenFromSystray (DisplayInfo *display, Window w)
 
     return NULL;
 }
-
