@@ -1,4 +1,4 @@
-/*      $Id: events.c,v 1.244 2004/09/05 20:57:46 olivier Exp $
+/*      $Id: events.c,v 1.277 2004/10/27 16:56:53 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -44,14 +44,15 @@
 #include "mywindow.h"
 #include "frame.h"
 #include "client.h"
+#include "stacking.h"
 #include "transients.h"
 #include "focus.h"
 #include "netwm.h"
 #include "menu.h"
 #include "hints.h"
 #include "startup_notification.h"
-#include "events.h"
 #include "compositor.h"
+#include "events.h"
 
 #define WIN_IS_BUTTON(win)      ((win == MYWINDOW_XWINDOW(c->buttons[HIDE_BUTTON])) || \
                                  (win == MYWINDOW_XWINDOW(c->buttons[CLOSE_BUTTON])) || \
@@ -74,11 +75,11 @@
 extern gboolean xfwm4_quit;
 extern gboolean xfwm4_reload;
 
-
 static guint raise_timeout = 0;
 static GdkAtom atom_rcfiles = GDK_NONE;
 static xfwmWindow menu_event_window;
 static int edge_scroll_x = 0;
+static int edge_scroll_y = 0;
 
 static void handleEvent (DisplayInfo *display_info, XEvent * ev);
 static void menu_callback (Menu * menu, MenuOp op, Window client_xwindow,
@@ -97,22 +98,104 @@ typedef enum
 }
 XfwmButtonClickType;
 
-static XfwmButtonClickType
-typeOfClick (DisplayInfo *display_info, Window w, XEvent * ev, gboolean allow_double_click)
+typedef struct _XfwmButtonClickData XfwmButtonClickData;
+struct _XfwmButtonClickData
 {
-    unsigned int button;
-    int xcurrent, ycurrent, x, y, total;
-    int g = GrabSuccess;
-    int clicks;
-    Time t0;
+    Time event_time;
+    Window w;
+    guint button;
+    gboolean allow_double_click;
+    guint clicks;
+    gint x;
+    gint y;
+    gint xcurrent;
+    gint ycurrent;
+    guint timeout;
+};
 
-    g_return_val_if_fail (display_info != NULL, XFWM_BUTTON_UNDEFINED);
+static gboolean
+typeOfClick_break (gpointer data)
+{
+    XfwmButtonClickData *passdata = (XfwmButtonClickData *) data;
+    if (passdata->timeout)
+    {
+        g_source_remove (passdata->timeout);
+        passdata->timeout = 0;
+    }
+
+    gtk_main_quit ();
+
+    return (TRUE);
+}
+
+static XfceFilterStatus
+typeOfClick_event_filter (XEvent * xevent, gpointer data)
+{
+    gboolean keep_going = TRUE;
+    XfceFilterStatus status = XEV_FILTER_STOP;
+    XfwmButtonClickData *passdata = (XfwmButtonClickData *) data;
+    
+    if ((xevent->type == ButtonRelease) || (xevent->type == ButtonPress))
+    {
+        if (xevent->xbutton.button == passdata->button)
+        {
+            passdata->clicks++;
+        }
+        passdata->event_time = xevent->xbutton.time;
+        if (((XfwmButtonClickType) passdata->clicks == XFWM_BUTTON_DOUBLE_CLICK)
+            || (!(passdata->allow_double_click) && 
+                 (XfwmButtonClickType) passdata->clicks == XFWM_BUTTON_CLICK))
+        {
+            keep_going = FALSE;
+        }
+    }
+    else if (xevent->type == MotionNotify)
+    {
+        passdata->xcurrent = xevent->xmotion.x_root;
+        passdata->ycurrent = xevent->xmotion.y_root;
+    }
+    else if ((xevent->type == DestroyNotify) || (xevent->type == UnmapNotify))
+    {
+        if (xevent->xany.window == passdata->w)
+        {
+            /* Discard, mark the click as undefined */
+            passdata->clicks = (guint) XFWM_BUTTON_UNDEFINED;
+            keep_going = FALSE;
+        }
+        status = XEV_FILTER_CONTINUE;
+    }
+    else
+    {
+        status = XEV_FILTER_CONTINUE;
+    }
+
+    if ((ABS (passdata->x - passdata->xcurrent) > 1) || 
+        (ABS (passdata->y - passdata->ycurrent) > 1) ||
+        (!keep_going))
+    {
+        TRACE ("event loop now finished");
+        typeOfClick_break (data);
+    }
+
+    return status;
+}
+
+static XfwmButtonClickType
+typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_double_click)
+{
+    int g = GrabSuccess;
+    DisplayInfo *display_info;
+    XfwmButtonClickData passdata;
+
+    g_return_val_if_fail (screen_info != NULL, XFWM_BUTTON_UNDEFINED);
     g_return_val_if_fail (ev != NULL, XFWM_BUTTON_UNDEFINED);
     g_return_val_if_fail (w != None, XFWM_BUTTON_UNDEFINED);
     
+    display_info = screen_info->display_info;    
     XFlush (display_info->dpy);
-    g = XGrabPointer (display_info->dpy, w, FALSE, DBL_CLICK_GRAB, GrabModeAsync,
-        GrabModeAsync, None, None, ev->xbutton.time);
+    g = XGrabPointer (display_info->dpy, screen_info->gnome_win, 
+                      FALSE, DBL_CLICK_GRAB, GrabModeAsync,
+                      GrabModeAsync, None, None, ev->xbutton.time);
     if (g != GrabSuccess)
     {
         TRACE ("grab failed in typeOfClick");
@@ -120,45 +203,42 @@ typeOfClick (DisplayInfo *display_info, Window w, XEvent * ev, gboolean allow_do
         return XFWM_BUTTON_UNDEFINED;
     }
 
-    button = ev->xbutton.button;
-    x = xcurrent = ev->xbutton.x_root;
-    y = ycurrent = ev->xbutton.y_root;
-    t0 = GDK_CURRENT_TIME;
-    total = 0;
-    clicks = 1;
+    passdata.button = ev->xbutton.button;
+    passdata.w = w;
+    passdata.x = ev->xbutton.x_root;
+    passdata.y = ev->xbutton.y_root;
+    passdata.xcurrent = passdata.x;
+    passdata.ycurrent = passdata.y;
+    passdata.clicks = 1;
+    passdata.allow_double_click = allow_double_click;
+    passdata.event_time = ev->xbutton.time;
+    passdata.timeout = g_timeout_add_full (0, display_info->dbl_click_time, 
+                                              (GtkFunction) typeOfClick_break, 
+                                              (gpointer) &passdata, NULL);
 
-    while ((ABS (x - xcurrent) < 2) && (ABS (y - ycurrent) < 2)
-        && (total < display_info->dbl_click_time)
-        && ((GDK_CURRENT_TIME - t0) < display_info->dbl_click_time))
-    {
-        g_usleep (10000);
-        total += 10;
-        if (XCheckMaskEvent (display_info->dpy, FocusChangeMask, ev))
-        {
-            handleEvent (display_info, ev);
-        }
-        if (XCheckMaskEvent (display_info->dpy, ButtonReleaseMask | ButtonPressMask, ev))
-        {
-            if (ev->xbutton.button == button)
-            {
-                clicks++;
-            }
-        }
-        if ((XfwmButtonClickType) clicks == XFWM_BUTTON_DOUBLE_CLICK
-            || (!allow_double_click
-                && (XfwmButtonClickType) clicks == XFWM_BUTTON_CLICK))
-        {
-            break;
-        }
-        if (XCheckMaskEvent (display_info->dpy, ButtonMotionMask | PointerMotionMask, ev))
-        {
-            xcurrent = ev->xmotion.x_root;
-            ycurrent = ev->xmotion.y_root;
-        }
-    }
-    XUngrabPointer (display_info->dpy, ev->xbutton.time);
+    TRACE ("entering typeOfClick loop");
+    xfce_push_event_filter (display_info->xfilter, typeOfClick_event_filter, &passdata);
+    gtk_main ();
+    xfce_pop_event_filter (display_info->xfilter);
+    TRACE ("leaving typeOfClick loop");
+
+    XUngrabPointer (display_info->dpy, passdata.event_time);
     XFlush (display_info->dpy);
-    return (XfwmButtonClickType) clicks;
+    return (XfwmButtonClickType) passdata.clicks;
+}
+
+static gboolean
+check_button_time (XButtonEvent *ev)
+{
+    static Time last_button_time = (Time) 0;
+    
+    if (last_button_time > ev->time)
+    {
+        return FALSE;
+    }
+
+    last_button_time = ev->time;
+    return TRUE;
 }
 
 static void
@@ -179,6 +259,7 @@ raise_cb (gpointer data)
 
     clear_timeout ();
     c = clientGetFocus ();
+    
     if (c)
     {
         clientRaise (c);
@@ -200,8 +281,8 @@ reset_timeout (ScreenInfo *screen_info)
 static void
 moveRequest (Client * c, XEvent * ev)
 {
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)))
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE)
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
         clientMove (c, ev);
     }
@@ -210,15 +291,13 @@ moveRequest (Client * c, XEvent * ev)
 static void
 resizeRequest (Client * c, int corner, XEvent * ev)
 {
-    clientSetFocus (c->screen_info, c, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
-
     if (FLAG_TEST_ALL (c->xfwm_flags,
             XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_IS_RESIZABLE))
     {
         clientResize (c, corner, ev);
     }
-    else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)))
+    else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE)
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
         clientMove (c, ev);
     }
@@ -246,7 +325,7 @@ spawn_shortcut (ScreenInfo *screen_info, int i)
 static void
 handleMotionNotify (DisplayInfo *display_info, XMotionEvent * ev)
 {
-    int msx, msy, max;
+    int msx, msy, maxx, maxy;
     ScreenInfo *screen_info = NULL;
     
     TRACE ("entering handleMotionNotify");
@@ -270,9 +349,10 @@ handleMotionNotify (DisplayInfo *display_info, XMotionEvent * ev)
     {
         msx = ev->x_root;
         msy = ev->y_root;
-        max = gdk_screen_get_width (screen_info->gscr) - 1;
+        maxx = gdk_screen_get_width (screen_info->gscr) - 1;
+        maxy = gdk_screen_get_height (screen_info->gscr) - 1;
 
-        if ((msx == 0) || (msx == max))
+        if ((msx == 0) || (msx == maxx))
         {
             edge_scroll_x++;
         }
@@ -280,18 +360,51 @@ handleMotionNotify (DisplayInfo *display_info, XMotionEvent * ev)
         {
             edge_scroll_x = 0;
         }
+        if ((msy == 0) || (msy == maxy))
+        {
+            edge_scroll_y++;
+        }
+        else
+        {
+            edge_scroll_y = 0;
+        }
+
         if (edge_scroll_x > screen_info->params->wrap_resistance)
         {
             edge_scroll_x = 0;
             if (msx == 0)
             {
-                XWarpPointer (display_info->dpy, None, screen_info->xroot, 0, 0, 0, 0, max - 10, msy);
-                workspaceSwitch (screen_info, screen_info->current_ws - 1, NULL);
+                if (workspaceMove (screen_info, 0, -1, NULL))
+                {
+                    XWarpPointer (display_info->dpy, None, screen_info->xroot, 0, 0, 0, 0, maxx - 10, msy);
+                }
             }
-            else if (msx == max)
+            else if (msx == maxx)
             {
-                XWarpPointer (display_info->dpy, None, screen_info->xroot, 0, 0, 0, 0, 10, msy);
-                workspaceSwitch (screen_info, screen_info->current_ws + 1, NULL);
+                if (workspaceMove (screen_info, 0, 1, NULL))
+                {
+                    XWarpPointer (display_info->dpy, None, screen_info->xroot, 0, 0, 0, 0, 10, msy);
+                }
+            }
+            while (XCheckWindowEvent(display_info->dpy, ev->window, PointerMotionMask, (XEvent *) ev))
+                ; /* Skip event */
+        }
+        if (edge_scroll_y > screen_info->params->wrap_resistance)
+        {
+            edge_scroll_y = 0;
+            if (msy == 0)
+            {
+                if (workspaceMove (screen_info, -1, 0, NULL))
+                {
+                    XWarpPointer (display_info->dpy, None, screen_info->xroot, 0, 0, 0, 0, msx, maxy - 10);
+                }
+            }
+            else if (msy == maxy)
+            {
+                if (workspaceMove (screen_info, 1, 0, NULL))
+                {
+                    XWarpPointer (display_info->dpy, None, screen_info->xroot, 0, 0, 0, 0, msx, 10);
+                }
             }
             while (XCheckWindowEvent(display_info->dpy, ev->window, PointerMotionMask, (XEvent *) ev))
                 ; /* Skip event */
@@ -397,6 +510,18 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
             case KEY_MOVE_PREV_WORKSPACE:
                 workspaceSwitch (screen_info, screen_info->current_ws - 1, c);
                 break;
+            case KEY_MOVE_UP_WORKSPACE:
+                workspaceMove (screen_info, -1, 0, c);
+                break;
+            case KEY_MOVE_DOWN_WORKSPACE:
+                workspaceMove (screen_info, 1, 0, c);
+                break;
+            case KEY_MOVE_LEFT_WORKSPACE:
+                workspaceMove (screen_info, 0, -1, c);
+                break;
+            case KEY_MOVE_RIGHT_WORKSPACE:
+                workspaceMove (screen_info, 0, 1, c);
+                break;
             case KEY_MOVE_WORKSPACE_1:
             case KEY_MOVE_WORKSPACE_2:
             case KEY_MOVE_WORKSPACE_3:
@@ -446,6 +571,18 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
         case KEY_PREV_WORKSPACE:
             workspaceSwitch (screen_info, screen_info->current_ws - 1, NULL);
             break;
+        case KEY_UP_WORKSPACE:
+            workspaceMove(screen_info, -1, 0, NULL);
+            break;
+        case KEY_DOWN_WORKSPACE:
+            workspaceMove(screen_info, 1, 0, NULL);
+            break;
+        case KEY_LEFT_WORKSPACE:
+            workspaceMove(screen_info, 0, -1, NULL);
+            break;
+        case KEY_RIGHT_WORKSPACE:
+            workspaceMove(screen_info, 0, 1, NULL);
+            break;
         case KEY_ADD_WORKSPACE:
             workspaceSetCount (screen_info, screen_info->workspace_count + 1);
             break;
@@ -492,16 +629,14 @@ edgeButton (Client * c, int part, XButtonEvent * ev)
     {
         XfwmButtonClickType tclick;
         ScreenInfo *screen_info = c->screen_info;
-        DisplayInfo *display_info = screen_info->display_info;
-
-        tclick = typeOfClick (display_info, c->frame, (XEvent *) ev, FALSE);
 
+        tclick = typeOfClick (screen_info, c->window, (XEvent *) ev, FALSE);
         if (tclick == XFWM_BUTTON_CLICK)
         {
             clientLower (c);
             clientPassGrabButton1 (NULL);
         }
-        else
+        else if (tclick != XFWM_BUTTON_UNDEFINED)
         {
             moveRequest (c, (XEvent *) ev);
         }
@@ -510,6 +645,10 @@ edgeButton (Client * c, int part, XButtonEvent * ev)
     {
         if (ev->button == Button1)
         {
+            if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
+            {
+                clientSetFocus (c->screen_info, c, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
+            }
             clientRaise (c);
             clientPassGrabButton1 (c);
         }
@@ -534,12 +673,15 @@ button1Action (Client * c, XButtonEvent * ev)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
 
-    clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+    if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
+    {
+        clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+    }
     clientRaise (c);
     clientPassGrabButton1 (c);
 
     memcpy(&copy_event, ev, sizeof(XEvent));
-    tclick = typeOfClick (display_info, c->frame, &copy_event, TRUE);
+    tclick = typeOfClick (screen_info, c->window, &copy_event, TRUE);
 
     if ((tclick == XFWM_BUTTON_DRAG)
         || (tclick == XFWM_BUTTON_CLICK_AND_DRAG))
@@ -599,15 +741,18 @@ titleButton (Client * c, int state, XButtonEvent * ev)
         XfwmButtonClickType tclick;
 
         memcpy(&copy_event, ev, sizeof(XEvent));
-        tclick = typeOfClick (display_info, c->frame, &copy_event, FALSE);
+        tclick = typeOfClick (screen_info, c->window, &copy_event, FALSE);
 
         if (tclick == XFWM_BUTTON_DRAG)
         {
             moveRequest (c, (XEvent *) ev);
         }
-        else
+        else if (tclick != XFWM_BUTTON_UNDEFINED)
         {
-            clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+            if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
+            {
+                clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+            }
             if (screen_info->params->raise_on_click)
             {
                 clientRaise (c);
@@ -647,7 +792,7 @@ rootScrollButton (DisplayInfo *display_info, XButtonEvent * ev)
     static Time lastscroll = (Time) 0;
     ScreenInfo *screen_info = NULL;
 
-    if ((ev->time - lastscroll) < 100)  /* ms */
+    if ((ev->time - lastscroll) < 25)  /* ms */
     {
         /* Too many events in too little time, drop this event... */
         return;
@@ -682,6 +827,13 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
 
     TRACE ("entering handleButtonPress");
 
+    /* Avoid treating the same event twice */
+    if (!check_button_time (ev))
+    {
+        TRACE ("ignoring ButtonPress event because it has been already handled");
+        return;
+    }
+
     /* Clear timeout */
     clear_timeout ();
 
@@ -724,7 +876,10 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
         {
             if (ev->button <= Button3)
             {
-                clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+                if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
+                {
+                    clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+                }
                 if (screen_info->params->raise_on_click)
                 {
                     clientRaise (c);
@@ -750,15 +905,18 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
                 XfwmButtonClickType tclick;
 
                 memcpy(&copy_event, ev, sizeof(XEvent));
-                tclick = typeOfClick (display_info, c->frame, &copy_event, TRUE);
+                tclick = typeOfClick (screen_info, c->window, &copy_event, TRUE);
 
                 if (tclick == XFWM_BUTTON_DOUBLE_CLICK)
                 {
                     clientClose (c);
                 }
-                else
+                else if (tclick != XFWM_BUTTON_UNDEFINED)
                 {
-                    clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+                    if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
+                    {
+                        clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+                    }
                     if (screen_info->params->raise_on_click)
                     {
                         clientRaise (c);
@@ -818,7 +976,10 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
                 {
                     clientPassGrabButton1 (c);
                 }
-                clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+                if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
+                {
+                    clientSetFocus (screen_info, c, ev->time, NO_FOCUS_FLAG);
+                }
                 if ((screen_info->params->raise_on_click) 
                     || !FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER))
                 {
@@ -854,7 +1015,8 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
         return;
     }
     
-    if ((ev->window == screen_info->xroot) && ((ev->button == Button4) || (ev->button == Button5)))
+    if ((ev->window == screen_info->xroot) && (screen_info->params->scroll_workspaces)
+            && ((ev->button == Button4) || (ev->button == Button5)))
     {
         rootScrollButton (display_info, ev);
     }
@@ -871,6 +1033,13 @@ handleButtonRelease (DisplayInfo *display_info, XButtonEvent * ev)
     ScreenInfo *screen_info = NULL;
     TRACE ("entering handleButtonRelease");
 
+    /* Avoid treating the same event twice */
+    if (!check_button_time (ev))
+    {
+        TRACE ("ignoring ButtonRelease event because it has been already handled");
+        return;
+    }
+
     /* Get the screen structure from the root of the event */
     screen_info = myDisplayGetScreenFromRoot (display_info, ev->root);
     if (!screen_info)
@@ -905,6 +1074,10 @@ handleDestroyNotify (DisplayInfo *display_info, XDestroyWindowEvent * ev)
         clientPassFocus (c->screen_info, c, c);
         clientUnframe (c, FALSE);
     }
+    else
+    {
+        compositorRemoveWindow (display_info, ev->window);
+    }
 }
 
 static void
@@ -964,6 +1137,10 @@ handleMapNotify (DisplayInfo *display_info, XMapEvent * ev)
             FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_MAP_PENDING);
         }
     }
+    else if (myDisplayGetScreenFromRoot (display_info, ev->event))
+    {
+        compositorMapWindow (display_info, ev->window);
+    }
 }
 
 static void
@@ -1035,6 +1212,10 @@ handleUnmapNotify (DisplayInfo *display_info, XUnmapEvent * ev)
             clientUnframe (c, FALSE);
         }
     }
+    else
+    {
+        compositorUnmapWindow (display_info, ev->window);
+    }
 }
 
 static void
@@ -1154,7 +1335,7 @@ handleConfigureRequest (DisplayInfo *display_info, XConfigureRequestEvent * ev)
             cx = frameX (c) + (frameWidth (c) / 2);
             cy = frameY (c) + (frameHeight (c) / 2);
 
-            monitor_nbr = gdk_screen_get_monitor_at_point (screen_info->gscr, cx, cy);
+            monitor_nbr = find_monitor_at_point (screen_info->gscr, cx, cy);
             gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
 
             wc.x = rect.x;
@@ -1276,10 +1457,13 @@ handleLeaveNotify (DisplayInfo *display_info, XCrossingEvent * ev)
     }
     
     if ((ev->window == MYWINDOW_XWINDOW (screen_info->sidewalk[0])) || 
-        (ev->window == MYWINDOW_XWINDOW (screen_info->sidewalk[1])))
+        (ev->window == MYWINDOW_XWINDOW (screen_info->sidewalk[1])) ||
+        (ev->window == MYWINDOW_XWINDOW (screen_info->sidewalk[2])) ||
+        (ev->window == MYWINDOW_XWINDOW (screen_info->sidewalk[3])))
     {
-        TRACE ("Reset edge_scroll_x");
+        TRACE ("Reset edge_scroll_x and edge_scroll_y");
         edge_scroll_x = 0;
+        edge_scroll_y = 0;
     }
 }
 
@@ -1288,6 +1472,8 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
 {
     ScreenInfo *screen_info = NULL;
     Client *c = NULL;
+    Client *last_raised = NULL;
+        
 
     TRACE ("entering handleFocusIn");
     TRACE ("handleFocusIn (0x%lx) mode = %s",
@@ -1346,7 +1532,16 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
         TRACE ("focus set to \"%s\" (0x%lx)", c->name, c->window);
         screen_info = c->screen_info;
         clientUpdateFocus (screen_info, c, FOCUS_SORT);
-        if (screen_info->params->raise_on_focus && !screen_info->params->click_to_focus)
+        last_raised = clientGetLastRaise (screen_info);
+        if ((screen_info->params->click_to_focus) && 
+            (screen_info->params->raise_on_click) && 
+            (last_raised != NULL) && (c != last_raised))
+        {
+            clientRaise (c);
+            clientPassGrabButton1 (c);
+        }
+        if ((screen_info->params->raise_on_focus) && 
+            !(screen_info->params->raise_on_click))
         {
             reset_timeout (screen_info);
         }
@@ -1483,6 +1678,7 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         }
         else if (ev->atom == wm_colormap_windows)
         {
+            TRACE ("client \"%s\" (0x%lx) has received a wm_colormap_windows notify", c->name, c->window);
             clientUpdateColormaps (c);
             if (c == clientGetFocus ())
             {
@@ -1491,11 +1687,21 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         }
         else if (ev->atom == net_wm_user_time)
         {
+            TRACE ("client \"%s\" (0x%lx) has received a net_wm_user_time notify", c->name, c->window);
             if (getNetWMUserTime (display_info->dpy, c->window, &c->user_time))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_HAS_USER_TIME);
             }
         }
+        else if (ev->atom == net_wm_opacity)
+        {
+            TRACE ("client \"%s\" (0x%lx) has received a net_wm_opacity notify", c->name, c->window);
+            if (!getOpacity (display_info->dpy, c->window, &c->opacity))
+            {
+                c->opacity =  NET_WM_OPAQUE;
+            }
+            compositorWindowSetOpacity (display_info, c->frame, c->opacity);
+        }
 #ifdef HAVE_STARTUP_NOTIFICATION
         else if (ev->atom == net_startup_id)
         {
@@ -1530,6 +1736,11 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         getGnomeDesktopMargins (display_info->dpy, screen_info->screen, screen_info->gnome_margins);
         workspaceUpdateArea (screen_info);
     }
+    else if (ev->atom == net_desktop_layout)
+    {
+        TRACE ("root has received a net_desktop_layout notify");
+        getDesktopLayout(display_info->dpy, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
+    }
 }
 
 static void
@@ -1537,17 +1748,15 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
 {
     ScreenInfo *screen_info = NULL;
     Client *c = NULL;
+    gboolean is_transient = FALSE;
 
     TRACE ("entering handleClientMessage");
 
-    /* Don't get surprised with the multiple "if (!clientIsTransientOrModal(c))" tests
-       xfwm4 really treats transient differently
-     */
-
     c = myDisplayGetClientFromWindow (display_info, ev->window, WINDOW);
     if (c)
     {
         screen_info = c->screen_info;
+        is_transient = clientIsValidTransientOrModal (c);
         
         if ((ev->message_type == wm_change_state) && (ev->format == 32) && (ev->data.l[0] == IconicState))
         {
@@ -1566,7 +1775,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         else if ((ev->message_type == win_layer) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_layer event", c->name, c->window);
-            if ((ev->data.l[0] != c->win_layer) && !clientIsTransientOrModal (c))
+            if ((ev->data.l[0] != c->win_layer) && !is_transient)
             {
                 clientSetLayer (c, ev->data.l[0]);
             }
@@ -1574,7 +1783,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         else if ((ev->message_type == win_workspace) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_workspace event", c->name, c->window);
-            if ((ev->data.l[0] != c->win_workspace) && !clientIsTransientOrModal (c))
+            if ((ev->data.l[0] != c->win_workspace) && !is_transient)
             {
                 clientSetWorkspace (c, ev->data.l[0], TRUE);
             }
@@ -1582,7 +1791,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         else if ((ev->message_type == net_wm_desktop) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_desktop event", c->name, c->window);
-            if (!clientIsTransientOrModal (c))
+            if (!is_transient)
             {
                 if (ev->data.l[0] == ALL_WORKSPACES)
                 {
@@ -1689,6 +1898,10 @@ handleShape (DisplayInfo *display_info, XShapeEvent * ev)
     {
         frameDraw (c, FALSE, TRUE);
     }
+    compositorUpdateWindow (display_info, ev->window, 
+                                          ev->width + ev->x, 
+                                          ev->height + ev->y, 
+                                          (ev->kind == ShapeBounding));
 }
 
 static void
@@ -1987,7 +2200,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
         if (FLAG_TEST(c->flags, CLIENT_FLAG_ABOVE))
         {
             ops |= MENU_OP_NORMAL;
-            if (clientIsTransientOrModal (c) || 
+            if (clientIsValidTransientOrModal (c) || 
                 FLAG_TEST (c->flags, CLIENT_FLAG_BELOW | CLIENT_FLAG_FULLSCREEN))
             {
                 insensitive |= MENU_OP_NORMAL;
@@ -1996,14 +2209,14 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
         else
         {
             ops |= MENU_OP_ABOVE;
-            if (clientIsTransientOrModal (c) || 
+            if (clientIsValidTransientOrModal (c) || 
                 FLAG_TEST (c->flags, CLIENT_FLAG_BELOW | CLIENT_FLAG_FULLSCREEN))
             {
                 insensitive |= MENU_OP_ABOVE;
             }
         }
 
-        if (clientIsTransientOrModal (c)
+        if (clientIsValidTransientOrModal (c)
             || !FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK)
             || FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
         {
@@ -2064,7 +2277,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
 }
 
 static gboolean
-set_reload (void)
+set_reload (GObject * obj, GdkEvent * ev, gpointer data)
 {
     TRACE ("setting reload flag so all prefs will be reread at next event loop");
     xfwm4_reload = TRUE;
@@ -2072,13 +2285,12 @@ set_reload (void)
 }
 
 static gboolean
-dbl_click_time (gpointer data)
+dbl_click_time_cb (GObject * obj, GdkEvent * ev, gpointer data)
 {
     DisplayInfo *display_info = (DisplayInfo *) data;
     GValue tmp_val = { 0, };
 
     g_return_val_if_fail (display_info, TRUE);
-    TRACE ("setting dbl_click_time");
     
     g_value_init (&tmp_val, G_TYPE_INT);
     if (gdk_setting_get ("gtk-double-click-time", &tmp_val))
@@ -2101,7 +2313,7 @@ client_event_cb (GtkWidget * widget, GdkEventClient * ev, gpointer data)
 
     if (ev->message_type == atom_rcfiles)
     {
-        set_reload ();
+        set_reload (G_OBJECT (widget), (GdkEvent *) ev, data);
     }
 
     return (FALSE);
@@ -2125,6 +2337,6 @@ initGtkCallbacks (ScreenInfo *screen_info)
         g_signal_connect (settings, "notify::gtk-font-name",
             G_CALLBACK (set_reload), NULL);
         g_signal_connect (settings, "notify::gtk-double-click-time",
-            G_CALLBACK (dbl_click_time), (gpointer) screen_info->display_info);
+            G_CALLBACK (dbl_click_time_cb), (gpointer) (screen_info->display_info));
     }
 }
