@@ -1,4 +1,5 @@
-/*
+/*      $Id: hints.h,v 1.53 2004/10/06 21:56:44 olivier Exp $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
@@ -15,7 +16,7 @@
  
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -31,6 +32,7 @@
 #include <X11/Xmd.h>
 #include <X11/Xatom.h>
 #include <glib.h>
+#include <gdk/gdk.h>
 
 #define MWM_HINTS_ELEMENTS                      3L
 
@@ -73,6 +75,7 @@
 
 #define WM_PROTOCOLS_TAKE_FOCUS                 (1L<<0)
 #define WM_PROTOCOLS_DELETE_WINDOW              (1L<<1)
+#define WM_PROTOCOLS_CONTEXT_HELP               (1L<<2)
 
 #define WIN_LAYER_DESKTOP                       0
 #define WIN_LAYER_BELOW                         2
@@ -95,6 +98,38 @@
 #define NET_WM_STATE_ADD                        1
 #define NET_WM_STATE_TOGGLE                     2
 
+#define NET_WM_ORIENTATION_HORZ                 0
+#define NET_WM_ORIENTATION_VERT                 1
+
+#define NET_WM_TOPLEFT                          0
+#define NET_WM_TOPRIGHT                         1
+#define NET_WM_BOTTOMRIGHT                      2
+#define NET_WM_BOTTOMLEFT                       3
+
+#define NET_WM_OPAQUE                           0xffffffff
+
+typedef struct
+{
+    unsigned long orientation;
+    unsigned long start;
+    unsigned long rows;
+    unsigned long cols;
+}
+NetWmDesktopLayout;
+
+#define LEFT                            0
+#define RIGHT                           1
+#define TOP                             2
+#define BOTTOM                          3
+#define LEFT_START_Y                    4
+#define LEFT_END_Y                      5
+#define RIGHT_START_Y                   6
+#define RIGHT_END_Y                     7
+#define TOP_START_X                     8
+#define TOP_END_X                       9
+#define BOTTOM_START_X                  10
+#define BOTTOM_END_X                    11
+
 typedef struct
 {
     unsigned long flags;
@@ -122,6 +157,7 @@ extern Atom wm_delete_window;
 extern Atom wm_protocols;
 extern Atom wm_state;
 extern Atom wm_takefocus;
+extern Atom wm_transient_for;
 extern Atom wm_window_role;
 
 /* freedesktop.org protocol */
@@ -133,7 +169,10 @@ extern Atom net_close_window;
 extern Atom net_current_desktop;
 extern Atom net_desktop_geometry;
 extern Atom net_desktop_viewport;
+extern Atom net_desktop_names;
+extern Atom net_desktop_layout;
 extern Atom net_number_of_desktops;
+extern Atom net_showing_desktop;
 extern Atom net_startup_id;
 extern Atom net_supported;
 extern Atom net_supporting_wm_check;
@@ -164,54 +203,75 @@ extern Atom net_wm_state_shaded;
 extern Atom net_wm_state_skip_pager;
 extern Atom net_wm_state_skip_taskbar;
 extern Atom net_wm_state_sticky;
+extern Atom net_wm_state_demands_attention;
 extern Atom net_wm_strut;
+extern Atom net_wm_strut_partial;
 extern Atom net_wm_window_type;
 extern Atom net_wm_window_type_desktop;
 extern Atom net_wm_window_type_dialog;
 extern Atom net_wm_window_type_dock;
 extern Atom net_wm_window_type_menu;
 extern Atom net_wm_window_type_normal;
-extern Atom net_wm_window_type_splashscreen;
+extern Atom net_wm_window_type_splash;
 extern Atom net_wm_window_type_toolbar;
 extern Atom net_wm_window_type_utility;
+extern Atom net_wm_user_time;
+extern Atom net_wm_opacity;
 extern Atom net_workarea;
 
-/* Systray similation for older KDE apps */
+/* KDE extension */
+extern Atom kde_net_wm_context_help;
 extern Atom kde_net_wm_system_tray_window_for;
+extern Atom kwm_win_icon;
+
+/* Systray similation for older KDE apps */
 extern Atom net_system_tray_manager;
 extern Atom net_system_tray_selection;
 extern Atom net_system_tray_opcode;
 
+/* Misc atoms */
+extern Atom misc_xrootpmap;
+extern Atom misc_xsetroot;
+extern Atom misc_pixmap;
+
 void initICCCMHints (Display *);
 unsigned long getWMState (Display *, Window);
 void setWMState (Display *, Window, unsigned long);
+void initMiscHints (Display *);
 void initMotifHints (Display *);
 PropMwmHints *getMotifHints (Display *, Window);
 unsigned int getWMProtocols (Display *, Window);
 void initGnomeHints (Display *);
-void initSystrayHints (Display *, int);
+void initKDEHints (Display *);
+Atom initSystrayHints (Display *, int);
 gboolean getHint (Display *, Window, Atom, long *);
 void setHint (Display *, Window, Atom, long);
+void getDesktopLayout (Display *, Window, int, NetWmDesktopLayout *);
 void getGnomeDesktopMargins (Display *, int, int *);
 void setGnomeProtocols (Display *, int, Window);
 void initNetHints (Display * dpy);
-void set_net_supported_hint (Display *, int, Window);
-gboolean get_atom_list (Display *, Window, Atom, Atom **, int *);
-gboolean get_cardinal_list (Display *, Window, Atom, unsigned long **, int *);
-void set_net_workarea (Display *, int, int, int *);
-void init_net_desktop_params (Display *, int, int);
+void setNetSupportedHint (Display *, int, Window);
+gboolean getAtomList (Display *, Window, Atom, Atom **, int *);
+gboolean getCardinalList (Display *, Window, Atom, unsigned long **, int *);
+void setNetWorkarea (Display *, int, int, int, int, int *);
+void initNetDesktopInfo (Display *, int, int, int, int);
 void set_utf8_string_hint (Display *, Window, Atom, const char *);
 void getTransientFor (Display *, int, Window, Window *);
 void getWindowName (Display *, Window, char **);
-gboolean get_utf8_string (Display *, Window, Atom, char **);
+gboolean getUTF8String (Display *, Window, Atom, char **, int *);
 void getWindowName (Display *, Window, char **);
 gboolean checkKdeSystrayWindow(Display *, Window);
 void sendSystrayReqDock(Display *, Window, Window);
-Window getSystrayWindow (Display *);
+Window getSystrayWindow (Display *, Atom);
 gboolean getWindowRole (Display *, Window, char **);
 Window getClientLeader (Display *, Window);
+gboolean getNetWMUserTime (Display *, Window, Time *);
 gboolean getClientID (Display *, Window, char **);
 gboolean getWindowCommand (Display *, Window, char ***, int *);
+gboolean getKDEIcon (Display *, Window, Pixmap *, Pixmap *);
+gboolean getRGBIconData (Display *, Window, unsigned long **, unsigned long *);
+gboolean getOpacity (Display *, Window, guint *);
+
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 gboolean getWindowStartupId (Display *, Window, char **);
 #endif
