@@ -1,4 +1,4 @@
-/*      $Id: main.c,v 1.110 2004/09/09 22:54:17 olivier Exp $
+/*      $Id: main.c,v 1.119 2004/10/24 13:32:37 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -69,10 +69,10 @@
 
 #ifdef HAVE_COMPOSITOR
 #define MAIN_EVENT_MASK BASE_EVENT_MASK|ExposureMask
-#else
+#else /* HAVE_COMPOSITOR */
 #define MAIN_EVENT_MASK BASE_EVENT_MASK
 #endif /* HAVE_COMPOSITOR */
-
+    
 static DisplayInfo *display_info = NULL;
 gboolean xfwm4_quit           = FALSE;
 gboolean xfwm4_reload         = FALSE;
@@ -101,28 +101,23 @@ cleanUp (void)
     display_info = NULL;
 }
 
-static char *build_session_filename(SessionClient *client_session)
+static char *
+build_session_filename(SessionClient *client_session)
 {
-    gchar *filename;
-    gchar *path;
-    gchar *file;
-    
-    path = (gchar *)xfce_get_userdir();
-    if (!g_file_test(path, G_FILE_TEST_IS_DIR) && mkdir(path, 0755) < 0) {
-            g_warning("Unable to create xfce user dir %s: %s",
-                       path, g_strerror(errno));
-            return NULL;
-    }
+    gchar *filename, *path, *file;
+    GError *error = NULL;
 
-    path = xfce_get_userfile("sessions", NULL);
-    if (!g_file_test(path, G_FILE_TEST_IS_DIR) && mkdir(path, 0755) < 0) 
+    path = xfce_resource_save_location (XFCE_RESOURCE_CACHE, "sessions", FALSE);
+    
+    if (!xfce_mkdirhier(path, 0700, &error)) 
     {
-        g_warning("Unable to create session dir %s: %s", 
-                  path, g_strerror(errno));
+        g_warning("Unable to create session dir %s: %s",
+                   path, error->message);
+        g_error_free (error);
         g_free (path);
         return NULL;
     }
-    
+
     file = g_strdup_printf("xfwm4-%s", client_session->given_client_id);
     filename = g_build_filename (path, file, NULL);
     g_free (file);
@@ -183,15 +178,179 @@ handleSignal (int sig)
             break;
         case SIGSEGV:
             cleanUp ();
-            g_error (_("%s: Segmentation fault"), g_get_prgname ());
+            g_error (_("%s: Segmentation fault"), PACKAGE);
             break;
         default:
             break;
     }
 }
 
+static void
+ensure_basedir_spec (void)
+{
+    char *new, *old, path[PATH_MAX];
+    GError *error = NULL;
+    GDir *gdir;
+    const char *name;
+
+    /* test if new directory is there */
+    
+    new = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, 
+                                       "xfce4" G_DIR_SEPARATOR_S "xfwm4", 
+                                       FALSE);
+    
+    if (g_file_test (new, G_FILE_TEST_IS_DIR))
+    {
+        g_free (new);
+        return;
+    }
+
+    if (!xfce_mkdirhier(new, 0700, &error)) 
+    {
+        g_warning("Unable to create config dir %s: %s",
+                  new, error->message);
+        g_error_free (error);
+        g_free (new);
+        return;
+    }
+
+    g_free (new);
+
+    /* copy xfwm4rc */
+    
+    old = xfce_get_userfile ("xfwm4rc", NULL);
+
+    if (g_file_test (old, G_FILE_TEST_EXISTS))
+    {
+        FILE *r, *w;
+
+        g_strlcpy (path, "xfce4/xfwm4/xfwm4rc", PATH_MAX);
+        new = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, FALSE);
+
+        r = fopen (old, "r");
+        w = fopen (new, "w");
+
+        g_free (new);
+        
+        if (w && r)
+        {
+            char c;
+            
+            while ((c = getc (r)) != EOF)
+                putc (c, w);
+        }
+
+        if (r)
+            fclose (r);
+        if (w)
+            fclose (w);
+    }
+    
+    g_free (old);
+
+    /* copy saved session data */
+    
+    new = xfce_resource_save_location (XFCE_RESOURCE_CACHE, "sessions", FALSE);
+
+    if (!xfce_mkdirhier(new, 0700, &error)) 
+    {
+        g_warning("Unable to create session dir %s: %s",
+                  new, error->message);
+        g_error_free (error);
+        g_free (new);
+        return;
+    }
+
+    old = xfce_get_userfile ("sessions", NULL);
+    gdir = g_dir_open (old, 0, NULL);
+
+    if (gdir)
+    {
+        while ((name = g_dir_read_name (gdir)) != NULL)
+        {
+            FILE *r, *w;
+
+            g_snprintf (path, PATH_MAX, "%s/%s", old, name);
+            r = fopen (path, "r");
+
+            g_snprintf (path, PATH_MAX, "%s/%s", new, name);
+            w = fopen (path, "w");
+
+            if (w && r)
+            {
+                char c;
+                
+                while ((c = getc (r)) != EOF)
+                    putc (c, w);
+            }
+
+            if (r)
+                fclose (r);
+            if (w)
+                fclose (w);
+        }
+
+        g_dir_close (gdir);
+    }
+
+    g_free (old);
+    g_free (new);
+}
+
+static void
+print_usage (void)
+{
+    g_print ("%s [--sm-client-id=ID] [--display=DISPLAY] \
+[--compositor=off|on|auto] [--daemon] \
+[--version|-V] [--help|-H]\n", PACKAGE);
+    exit (0);
+}
+
+static void
+print_version (void)
+{
+    g_print ("\tThis is %s version %s for Xfce %s\n", 
+                    PACKAGE, VERSION, xfce_version_string());
+    g_print ("\tbuilt with GTK+-%d.%d.%d, ", 
+                    GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
+    g_print ("linked with GTK+-%d.%d.%d.\n", 
+                    gtk_major_version, gtk_minor_version, gtk_micro_version);
+    exit (0);
+}
+
+static gint
+parse_compositor (const gchar *s)
+{
+    gchar *buffer, *rvalue;
+    gboolean retval = 2;
+
+    rvalue = strrchr (s, '=');
+    if (rvalue)
+    {
+        rvalue++;
+        if (!strcmp (rvalue, "off"))
+        {
+            retval=0;
+        }
+        else if (!strcmp (rvalue, "auto"))
+        {
+            retval=1;
+        }
+        else if (!strcmp (rvalue, "on"))
+        {
+            retval=2;
+        }
+        else
+        {
+            g_warning ("%s: Unrecognized compositor option \"%s\"", PACKAGE, rvalue);
+        }
+    }
+
+    return retval;
+}
+
 static int
-initialize (int argc, char **argv)
+initialize (int argc, char **argv, gint compositor_mode)
 {
     struct sigaction act;
     long ws;
@@ -202,25 +361,24 @@ initialize (int argc, char **argv)
     
     xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
-    gtk_set_locale ();
     gtk_init (&argc, &argv);
 
     DBG ("xfwm4 starting, using GTK+-%d.%d.%d", gtk_major_version, 
          gtk_minor_version, gtk_micro_version);
 
+    ensure_basedir_spec ();
+    
     initMenuEventWin ();
     clientClearFocus ();
 
     display_info = myDisplayInit (gdk_display_get_default ());
-
-#ifdef HAVE_COMPOSITOR
-    if (!display_info->enable_compositor)
+    
+    /* Disabling compositor from command line */
+    if (!compositor_mode)
     {
-        g_warning (_("%s: Cannot find required extensions " 
-                     "for compositing manager"), g_get_prgname ());
+        display_info->enable_compositor = FALSE;
     }
-#endif /* HAVE_COMPOSITOR */
-
+    
     initModifiers (display_info->dpy);
 
     act.sa_handler = handleSignal;
@@ -250,6 +408,11 @@ initialize (int argc, char **argv)
             return -2;
         }
         
+        if (compositor_mode)
+        {
+            compositorManageScreen (screen_info, (compositor_mode == 2));
+        }
+
         sn_init_display (screen_info);
         myDisplayAddScreen (display_info, screen_info);
         setGnomeProtocols (display_info->dpy, screen_info->screen, screen_info->gnome_win);
@@ -271,7 +434,10 @@ initialize (int argc, char **argv)
         clientFrameAll (screen_info);
         
         initGtkCallbacks (screen_info);
+
+        XDefineCursor (display_info->dpy, screen_info->xroot, myDisplayGetCursorRoot(display_info));
     }
+
     /* No screen to manage, give up */
     if (!display_info->nb_screens)
     {
@@ -299,38 +465,40 @@ main (int argc, char **argv)
 {
     int i;
     gboolean daemon_mode = FALSE;
+    gint compositor = 2;
     int status;
 
-    if (argc > 1 && (!strcmp(argv[1], "--version") || !strcmp(argv[1], "-V")))
-    {
-        g_print ("\tThis is %s version %s for Xfce %s\n", 
-                      PACKAGE, VERSION, xfce_version_string());
-        g_print ("\tbuilt with GTK+-%d.%d.%d, ", 
-                      GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
-        g_print ("linked with GTK+-%d.%d.%d.\n", 
-                      gtk_major_version, gtk_minor_version, gtk_micro_version);
-        exit (0);
-    }
-
     for (i = 1; i < argc; i++)
     {
         if (!strcmp (argv[i], "--daemon"))
         {
             daemon_mode = TRUE;
         }
+        else if (!strncmp (argv[i], "--compositor=", strlen ("--compositor=")))
+        {
+            compositor = parse_compositor (argv[i]);
+        }
+        else if (!strcmp(argv[i], "--version") || !strcmp(argv[i], "-V"))
+        {
+            print_version ();
+        }
+        else if (!strcmp(argv[i], "--help") || !strcmp(argv[i], "-H"))
+        {
+            print_usage ();
+        }
     }
 
-    status = initialize (argc, argv);
+    status = initialize (argc, argv, compositor);
     switch (status)
     {
         case -1:
-            g_warning (_("%s: Another Window Manager is already running"),
-                g_get_prgname ());
+            g_warning ("%s: Another Window Manager is already running",
+                PACKAGE);
             exit (1);
             break;
         case -2:
-            g_warning (_("%s: Missing data from default files"),
-                g_get_prgname ());
+            g_warning ("%s: Missing data from default files",
+                PACKAGE);
             exit (1);
             break;
         case 0:
@@ -339,7 +507,7 @@ main (int argc, char **argv)
 #ifdef HAVE_DAEMON
                 if (daemon(TRUE, TRUE) < 0) 
                 {
-                        g_warning(_("%s: Failed to enter daemon mode: %s"),
+                        g_warning("%s: Failed to enter daemon mode: %s",
                                         g_get_prgname(), g_strerror(errno));
                         exit(EXIT_FAILURE);
                 }
@@ -347,7 +515,7 @@ main (int argc, char **argv)
                 switch (fork ())
                 {
                     case -1:
-                        g_warning (_("%s: Failed to create new process: %s"),
+                        g_warning ("%s: Failed to create new process: %s",
                                                                 g_get_prgname(), g_strerror(errno));
                         exit (1);
                         break;
@@ -368,7 +536,7 @@ main (int argc, char **argv)
             gtk_main ();
             break;
         default:
-            g_warning (_("%s: Unknown error occured"), g_get_prgname ());
+            g_warning ("%s: Unknown error occured", PACKAGE);
             exit (1);
             break;
     }
