@@ -1,4 +1,5 @@
-/*
+/*      $Id: menu.c,v 1.46 2004/08/22 22:18:50 olivier Exp $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
@@ -14,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -22,18 +23,14 @@
 #include <config.h>
 #endif
 
-#ifdef GDK_MULTIHEAD_SAFE
-#undef GDK_MULTIHEAD_SAFE
-#endif
-
+#include <X11/Xlib.h>
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <X11/Xlib.h>
 #include <unistd.h>
-#include <libxfce4util/debug.h>
-#include <libxfce4util/i18n.h>
+#include <string.h>
+#include <libxfce4util/libxfce4util.h> 
 #include <libxfcegui4/libxfcegui4.h>
 #include "menu.h"
 
@@ -48,6 +45,9 @@ static MenuItem menuitems[] = {
     {MENU_OP_UNSHADE, "gtk-goto-bottom", N_("Un_shade")},
     {MENU_OP_STICK, "gtk-add", N_("S_tick")},
     {MENU_OP_UNSTICK, "gtk-remove", N_("Uns_tick")},
+    {MENU_OP_CONTEXT_HELP, "gtk-help", N_("Context _help")},
+    {MENU_OP_ABOVE, NULL, N_("Always on top")},
+    {MENU_OP_NORMAL, "gtk-apply", N_("Always on top")},
     {MENU_OP_WORKSPACES, NULL, N_("Send to...")},
     {0, NULL, NULL},
     {MENU_OP_DELETE, "gtk-close", N_("_Close")},
@@ -60,7 +60,7 @@ static MenuItem menuitems[] = {
     {MENU_OP_RESTART, "gtk-refresh", N_("Restart")},
 };
 
-static GtkToXEventFilterStatus
+static XfceFilterStatus
 menu_filter (XEvent * xevent, gpointer data)
 {
     switch (xevent->type)
@@ -117,19 +117,22 @@ popup_position_func (GtkMenu * menu, gint * x, gint * y, gboolean * push_in,
 static gboolean
 activate_cb (GtkWidget * menuitem, gpointer data)
 {
-    MenuData *md;
+    MenuData *menudata;
 
     TRACE ("entering activate_cb");
     g_return_val_if_fail (GTK_IS_WIDGET (menuitem), FALSE);
 
     menu_open = NULL;
 
-    md = data;
+    menudata = data;
 
     TRACE ("deactivating menu_filter");
-    popEventFilter ();
-    (*md->menu->func) (md->menu, md->op, md->client_xwindow, md->menu->data,
-        md->data);
+    xfce_pop_event_filter (menudata->menu->filter_setup);
+    (*menudata->menu->func) (menudata->menu, 
+                             menudata->op, 
+                             menudata->client_xwindow, 
+                             menudata->menu->data,
+                             menudata->data);
     return (FALSE);
 }
 
@@ -142,50 +145,67 @@ menu_closed (GtkMenu * widget, gpointer data)
     menu = data;
     menu_open = NULL;
     TRACE ("deactivating menu_filter");
-    popEventFilter ();
+    xfce_pop_event_filter (menu->filter_setup);
     (*menu->func) (menu, 0, None, menu->data, NULL);
     return (FALSE);
 }
 
 static GtkWidget *
-menu_workspace (Menu * menu, MenuOp insensitive, gint ws, gint nws)
+menu_workspace (Menu * menu, MenuOp insensitive, gint ws, gint nws, gchar *wsn, gint wsnl)
 {
     gint i;
     GtkWidget *menu_widget;
-    GtkWidget *mi;
-    MenuData *md;
+    GtkWidget *menuitem;
+    MenuData *menudata;
     gchar *name;
+    gchar *ptr = wsn;
 
     menu_widget = gtk_menu_new ();
+    gtk_menu_set_screen (GTK_MENU (menu->menu), menu->screen);
 
     for (i = 0; i < nws; i++)
     {
-        name = g_strdup_printf (_("Workspace %i"), i + 1);
-        mi = gtk_check_menu_item_new_with_label (name);
-        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mi), (i == ws));
-        gtk_widget_show (mi);
+        if (ptr && *ptr)
+        {
+            name = g_strdup_printf (_("Workspace %i (%s)"), i + 1, ptr);
+            if (ptr - wsn + 1 < wsnl)
+            {
+                ptr += strlen (ptr) + 1;
+            }
+            else
+            {
+                ptr = NULL;
+            }
+        }
+        else
+        {
+            name = g_strdup_printf (_("Workspace %i"), i + 1);
+        }
+        menuitem = gtk_check_menu_item_new_with_label (name);
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), (i == ws));
+        gtk_widget_show (menuitem);
         if (insensitive & MENU_OP_WORKSPACES)
         {
-            gtk_widget_set_sensitive (mi, FALSE);
+            gtk_widget_set_sensitive (menuitem, FALSE);
         }
         g_free (name);
 
-        md = g_new (MenuData, 1);
-        md->menu = menu;
-        md->op = MENU_OP_WORKSPACES;
-        md->client_xwindow = None;
-        md->data = GINT_TO_POINTER (i);
-        menu_item_connect (mi, md);
+        menudata = g_new (MenuData, 1);
+        menudata->menu = menu;
+        menudata->op = MENU_OP_WORKSPACES;
+        menudata->client_xwindow = None;
+        menudata->data = GINT_TO_POINTER (i);
+        menu_item_connect (menuitem, menudata);
 
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu_widget), mi);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu_widget), menuitem);
     }
 
     return (menu_widget);
 }
 
 Menu *
-menu_default (MenuOp ops, MenuOp insensitive, MenuFunc func, gint ws,
-    gint nws, gpointer data)
+menu_default (GdkScreen *gscr, MenuOp ops, MenuOp insensitive, MenuFunc func, gint ws,
+    gint nws, gchar *wsn, gint wsnl, XfceFilterSetup *filter_setup, gpointer data)
 {
     int i;
     Menu *menu;
@@ -193,20 +213,23 @@ menu_default (MenuOp ops, MenuOp insensitive, MenuFunc func, gint ws,
     TRACE ("entering menu_new");
     menu = g_new (Menu, 1);
     menu->func = func;
+    menu->filter_setup = filter_setup;
     menu->data = data;
     menu->ops = ops;
     menu->insensitive = insensitive;
+    menu->screen = gscr;
     menu->menu = gtk_menu_new ();
+    gtk_menu_set_screen (GTK_MENU (menu->menu), menu->screen);
 
     i = 0;
     while (i < (int) (sizeof (menuitems) / sizeof (MenuItem)))
     {
         if ((ops & menuitems[i].op) || (menuitems[i].op == MENU_OP_SEPARATOR))
         {
-            GtkWidget *mi;
+            GtkWidget *menuitem;
             GtkWidget *image;
             GtkWidget *ws_menu;
-            MenuData *md;
+            MenuData *menudata;
             const gchar *label;
 
             label = _(menuitems[i].label);
@@ -214,53 +237,52 @@ menu_default (MenuOp ops, MenuOp insensitive, MenuFunc func, gint ws,
             switch (menuitems[i].op)
             {
                 case MENU_OP_SEPARATOR:
-                    mi = gtk_separator_menu_item_new ();
+                    menuitem = gtk_separator_menu_item_new ();
                     break;
                 case MENU_OP_WORKSPACES:
-                    mi = gtk_menu_item_new_with_mnemonic (label);
+                    menuitem = gtk_menu_item_new_with_mnemonic (label);
                     if (insensitive & menuitems[i].op)
                     {
-                        gtk_widget_set_sensitive (mi, FALSE);
+                        gtk_widget_set_sensitive (menuitem, FALSE);
                     }
-                    ws_menu = menu_workspace (menu, insensitive, ws, nws);
-                    gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), ws_menu);
+                    ws_menu = menu_workspace (menu, insensitive, ws, nws, wsn, wsnl);
+                    gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), ws_menu);
 
-                    md = g_new (MenuData, 1);
-                    md->menu = menu;
-                    md->op = menuitems[i].op;
-                    md->client_xwindow = None;
-                    md->data = NULL;
+                    menudata = g_new (MenuData, 1);
+                    menudata->menu = menu;
+                    menudata->op = menuitems[i].op;
+                    menudata->client_xwindow = None;
+                    menudata->data = NULL;
                     break;
                 default:
                     if (menuitems[i].image_name)
                     {
-                        mi = gtk_image_menu_item_new_with_mnemonic (label);
+                        menuitem = gtk_image_menu_item_new_with_mnemonic (label);
                         image =
                             gtk_image_new_from_stock (menuitems[i].image_name,
                             GTK_ICON_SIZE_MENU);
                         gtk_widget_show (image);
-                        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM
-                            (mi), image);
+                        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
 
                     }
                     else
                     {
-                        mi = gtk_menu_item_new_with_mnemonic (label);
+                        menuitem = gtk_menu_item_new_with_mnemonic (label);
                     }
                     if (insensitive & menuitems[i].op)
                     {
-                        gtk_widget_set_sensitive (mi, FALSE);
+                        gtk_widget_set_sensitive (menuitem, FALSE);
                     }
-                    md = g_new (MenuData, 1);
-                    md->menu = menu;
-                    md->op = menuitems[i].op;
-                    md->client_xwindow = None;
-                    md->data = NULL;
-                    menu_item_connect (mi, md);
+                    menudata = g_new (MenuData, 1);
+                    menudata->menu = menu;
+                    menudata->op = menuitems[i].op;
+                    menudata->client_xwindow = None;
+                    menudata->data = NULL;
+                    menu_item_connect (menuitem, menudata);
                     break;
             }
-            gtk_menu_shell_append (GTK_MENU_SHELL (menu->menu), mi);
-            gtk_widget_show (mi);
+            gtk_menu_shell_append (GTK_MENU_SHELL (menu->menu), menuitem);
+            gtk_widget_show (menuitem);
         }
         ++i;
     }
@@ -325,7 +347,7 @@ menu_check_and_close (void)
 }
 
 static gboolean
-grab_available (guint32 timestamp)
+grab_available (GdkWindow *win, guint32 timestamp)
 {
     GdkEventMask mask =
         GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
@@ -338,9 +360,8 @@ grab_available (guint32 timestamp)
 
     TRACE ("entering grab_available");
 
-    g1 = gdk_pointer_grab (getGdkEventWindow (), TRUE, mask, NULL, NULL,
-        timestamp);
-    g2 = gdk_keyboard_grab (getGdkEventWindow (), TRUE, timestamp);
+    g1 = gdk_pointer_grab (win, TRUE, mask, NULL, NULL, timestamp);
+    g2 = gdk_keyboard_grab (win, TRUE, timestamp);
 
     while ((i++ < 100) && (grab_failed = ((g1 != GDK_GRAB_SUCCESS)
                 || (g2 != GDK_GRAB_SUCCESS))))
@@ -349,12 +370,11 @@ grab_available (guint32 timestamp)
         g_usleep (100);
         if (g1 != GDK_GRAB_SUCCESS)
         {
-            g1 = gdk_pointer_grab (getGdkEventWindow (), TRUE, mask, NULL,
-                NULL, timestamp);
+            g1 = gdk_pointer_grab (win, TRUE, mask, NULL, NULL, timestamp);
         }
         if (g2 != GDK_GRAB_SUCCESS)
         {
-            g2 = gdk_keyboard_grab (getGdkEventWindow (), TRUE, timestamp);
+            g2 = gdk_keyboard_grab (win, TRUE, timestamp);
         }
     }
 
@@ -387,7 +407,7 @@ menu_popup (Menu * menu, int root_x, int root_y, int button,
 
     if (!menu_check_and_close ())
     {
-        if (!grab_available (timestamp))
+        if (!grab_available (gdk_screen_get_root_window (menu->screen), timestamp))
         {
             g_free (pt);
             TRACE ("Cannot get grab on pointer/keyboard, cancel.");
@@ -395,18 +415,17 @@ menu_popup (Menu * menu, int root_x, int root_y, int button,
         }
         TRACE ("opening new menu");
         menu_open = menu->menu;
-        pushEventFilter (menu_filter, NULL);
+        xfce_push_event_filter (menu->filter_setup, menu_filter, NULL);
         gtk_menu_popup (GTK_MENU (menu->menu), NULL, NULL,
             popup_position_func, pt, 0, timestamp);
 
         if (!GTK_MENU_SHELL (GTK_MENU (menu->menu))->have_xgrab)
         {
             gdk_beep ();
-            g_message (_("%s: GtkMenu failed to grab the pointer\n"),
-                g_get_prgname ());
+            g_message (_("%s: GtkMenu failed to grab the pointer\n"), g_get_prgname ());
             gtk_menu_popdown (GTK_MENU (menu->menu));
             menu_open = NULL;
-            popEventFilter ();
+            xfce_pop_event_filter (menu->filter_setup);
             return FALSE;
         }
     }
