@@ -1,4 +1,5 @@
-/*
+/*      $Id: misc.h,v 1.30 2004/10/17 08:41:54 olivier Exp $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
@@ -14,13 +15,10 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
-#ifndef INC_MISC_H
-#define INC_MISC_H
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -28,16 +26,17 @@
 #include <sys/types.h>
 #include <X11/Xlib.h>
 #include <glib.h>
+#include "screen.h"
 
-void getMouseXY (Window, int *, int *);
-Window getMouseWindow (Window);
-GC createGC (Colormap, char *, int, XFontStruct *, int, gboolean);
-void sendClientMessage (Window, Atom, Atom, Time);
-void MyXGrabServer (void);
-void MyXUngrabServer (void);
-gboolean MyCheckWindow(Window);
-Window setTmpEventWin (int, int, unsigned int, unsigned int, long);
-void removeTmpEventWin (Window);
-void placeSidewalks(gboolean);
+#ifndef INC_MISC_H
+#define INC_MISC_H
+
+void getMouseXY (ScreenInfo *, Window, int *, int *);
+Window getMouseWindow (ScreenInfo *, Window);
+GC createGC (ScreenInfo *, char *, int, XFontStruct *, int, gboolean);
+void sendClientMessage (ScreenInfo *, Window, Atom, Atom, Time);
+gboolean checkWindowOnRoot (ScreenInfo *, Window);
+void placeSidewalks(ScreenInfo *, gboolean);
+gint find_monitor_at_point (GdkScreen *screen, gint x, gint y);
 
 #endif /* INC_MISC_H */
