@@ -1,4 +1,4 @@
-/*      $Id: screen.c,v 1.7 2004/09/25 21:49:33 olivier Exp $
+/*      $Id: screen.c,v 1.8 2004/10/13 20:50:55 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -172,9 +172,6 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
         xfwmPixmapInit (screen_info, &screen_info->title[i][ACTIVE]);
         xfwmPixmapInit (screen_info, &screen_info->title[i][INACTIVE]);
     }
-    compositorManageScreen (screen_info);
-
-    XDefineCursor (display_info->dpy, screen_info->xroot, myDisplayGetCursorRoot(display_info));
 
     return (screen_info);
 }
