@@ -1,4 +1,4 @@
-/*      $Id: settings.c,v 1.104 2004/09/08 15:59:31 olivier Exp $
+/*      $Id: settings.c,v 1.115 2004/10/24 13:32:37 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
 #define CHANNEL1 "xfwm4"
 #define CHANNEL2 "margins"
 #define CHANNEL3 "workspaces"
+#define CHANNEL4 "xfwm4_keys"
 #define TOINT(x) (x ? atoi(x) : 0)
 #define DEFAULT_KEYTHEME "Default"
 #define KEYTHEMERC "keythemerc"
@@ -154,6 +155,10 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     {
                         screen_info->params->raise_on_focus = setting->data.v_int;
                     }
+                    else if (!strcmp (name, "Xfwm/PreventFocusStealing"))
+                    {
+                        screen_info->params->prevent_focus_stealing = setting->data.v_int;
+                    }
                     else if (!strcmp (name, "Xfwm/RaiseDelay"))
                     {
                         screen_info->params->raise_delay = setting->data.v_int;
@@ -203,10 +208,6 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     {
                         reloadScreenSettings (screen_info, NO_UPDATE_FLAG);
                     }
-                    else if (!strcmp (name, "Xfwm/KeyThemeName"))
-                    {
-                        reloadScreenSettings (screen_info, UPDATE_KEYGRABS);
-                    }
                     else if (!strcmp (name, "Xfwm/ThemeName"))
                     {
                         reloadScreenSettings (screen_info, UPDATE_GRAVITY | UPDATE_CACHE);
@@ -290,6 +291,30 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                 break;
         }
     }
+    else if (!g_ascii_strcasecmp (CHANNEL4, channel_name))
+    {
+        switch (action)
+        {
+            case MCS_ACTION_NEW:
+                /* The following is to reduce initial startup time and reloads */
+                if (!screen_info->mcs_initted)
+                {
+                    return;
+                }
+            case MCS_ACTION_CHANGED:
+                if (setting->type == MCS_TYPE_STRING)
+                {
+                    if (!strcmp (name, "Xfwm/KeyThemeName"))
+                    {
+                        reloadScreenSettings (screen_info, UPDATE_KEYGRABS);
+                    }
+                }
+                break;
+            case MCS_ACTION_DELETED:
+            default:
+                break;
+        }
+    }
 }
 
 static GdkFilterReturn
@@ -333,23 +358,37 @@ watch_cb (Window window, Bool is_start, long mask, void *cb_data)
 static void
 loadRcData (ScreenInfo *screen_info, Settings rc[])
 {
-    const gchar *homedir = xfce_get_userdir ();
-    gchar *keytheme;
+    gchar *homedir;
     gchar *keythemevalue;
+    gchar *keytheme;
+    gchar *system_keytheme;
+
 
     if (!parseRc ("defaults", PACKAGE_DATADIR, rc))
     {
-        g_warning (_("%s: Missing defaults file"), g_get_prgname ());
+        g_warning ("%s: Missing defaults file", g_get_prgname ());
         exit (1);
     }
     keythemevalue = getValue ("keytheme", rc);
     if (keythemevalue)
     {
+        system_keytheme = getSystemThemeDir ();
+        parseRc (KEYTHEMERC, system_keytheme, rc);
+        
         keytheme = getThemeDir (keythemevalue, KEYTHEMERC);
-        parseRc (KEYTHEMERC, keytheme, rc);
-        g_free (keytheme);
+        if (keytheme)
+        {
+            /* If there is a custom key theme, merge it with system defaults */
+            parseRc (KEYTHEMERC, keytheme, rc);
+            g_free (keytheme);
+        }
+        g_free (system_keytheme);
     }
+    homedir = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, 
+                                           "xfce4" G_DIR_SEPARATOR_S "xfwm4",
+                                           FALSE);
     parseRc ("xfwm4rc", homedir, rc);
+    g_free (homedir);
 }
 
 static void
@@ -376,6 +415,12 @@ loadMcsData (ScreenInfo *screen_info, Settings rc[])
             setBooleanValueFromInt ("raise_on_focus", setting->data.v_int, rc);
             mcs_setting_free (setting);
         }
+        if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/PreventFocusStealing", CHANNEL1,
+                &setting) == MCS_SUCCESS)
+        {
+            setBooleanValueFromInt ("prevent_focus_stealing", setting->data.v_int, rc);
+            mcs_setting_free (setting);
+        }
         if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/RaiseDelay", CHANNEL1,
                 &setting) == MCS_SUCCESS)
         {
@@ -449,12 +494,6 @@ loadMcsData (ScreenInfo *screen_info, Settings rc[])
             setValue ("theme", setting->data.v_string, rc);
             mcs_setting_free (setting);
         }
-        if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/KeyThemeName", CHANNEL1,
-                &setting) == MCS_SUCCESS)
-        {
-            setValue ("keytheme", setting->data.v_string, rc);
-            mcs_setting_free (setting);
-        }
         if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/ButtonLayout", CHANNEL1,
                 &setting) == MCS_SUCCESS)
         {
@@ -503,6 +542,12 @@ loadMcsData (ScreenInfo *screen_info, Settings rc[])
             setIntValueFromInt ("workspace_count", setting->data.v_int, rc);
             mcs_setting_free (setting);
         }
+        if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/KeyThemeName", CHANNEL4,
+                &setting) == MCS_SUCCESS)
+        {
+            setValue ("keytheme", setting->data.v_string, rc);
+            mcs_setting_free (setting);
+        }
     }
 }
 
@@ -546,14 +591,23 @@ loadTheme (ScreenInfo *screen_info, Settings rc[])
     theme = getThemeDir (getValue ("theme", rc), THEMERC);
     parseRc (THEMERC, theme, rc);
 
+    screen_info->params->shadow_delta_x = 
+        abs (TOINT (getValue ("shadow_delta_x", rc)));
+    screen_info->params->shadow_delta_y = 
+        abs (TOINT (getValue ("shadow_delta_y", rc)));
+    screen_info->params->shadow_delta_width = 
+        abs (TOINT (getValue ("shadow_delta_width", rc)));
+    screen_info->params->shadow_delta_height = 
+        abs (TOINT (getValue ("shadow_delta_height", rc)));
+
     for (i = 0; i < 20; i++)
     {
         colsym[i].name = rc[i].option;
         colsym[i].value = rc[i].value;
     }
 
-    g_value_init (&tmp_val, G_TYPE_INT);
     display_info->dbl_click_time = abs (TOINT (getValue ("dbl_click_time", rc)));
+    g_value_init (&tmp_val, G_TYPE_INT);
     if (gdk_setting_get ("gtk-double-click-time", &tmp_val))
     {
         display_info->dbl_click_time = abs (g_value_get_int (&tmp_val));
@@ -838,14 +892,14 @@ loadKeyBindings (ScreenInfo *screen_info, Settings rc[])
         keytheme = getThemeDir (keythemevalue, KEYTHEMERC);
         if (!parseRc (KEYTHEMERC, keytheme, rc))
         {
-            g_warning (_("%s: specified key theme \"%s\" missing, using default"),
+            g_warning ("%s: specified key theme \"%s\" missing, using default",
                        g_get_prgname (), keythemevalue);
         }
         g_free (keytheme);
 
         if (!checkRc (rc))
         {
-            g_warning (_("%s: Missing values in defaults file"), g_get_prgname ());
+            g_warning ("%s: Missing values in defaults file", g_get_prgname ());
             return FALSE;
         }
     }
@@ -905,13 +959,21 @@ loadKeyBindings (ScreenInfo *screen_info, Settings rc[])
     parseKeyString (dpy, &screen_info->params->keys[KEY_RAISE_WINDOW], getValue ("raise_window_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_LOWER_WINDOW], getValue ("lower_window_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_TOGGLE_FULLSCREEN], getValue ("fullscreen_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_UP_WORKSPACE], getValue ("up_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_DOWN_WORKSPACE], getValue ("down_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_LEFT_WORKSPACE], getValue ("left_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_RIGHT_WORKSPACE], getValue ("right_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_UP_WORKSPACE], getValue ("move_window_up_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_DOWN_WORKSPACE], getValue ("move_window_down_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_LEFT_WORKSPACE], getValue ("move_window_left_workspace_key", rc));
+    parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_RIGHT_WORKSPACE], getValue ("move_window_right_workspace_key", rc));
 
     ungrabKeys (dpy, screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_CYCLE_WINDOWS], screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_NEXT_WORKSPACE], screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_PREV_WORKSPACE], screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_ADD_WORKSPACE], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_NEXT_WORKSPACE], screen_info->gnome_win);
+    grabKey (dpy, &screen_info->params->keys[KEY_DEL_WORKSPACE], screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_1], screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_2], screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_3], screen_info->gnome_win);
@@ -931,6 +993,11 @@ loadKeyBindings (ScreenInfo *screen_info, Settings rc[])
     grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_8], screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_9], screen_info->gnome_win);
     grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_10], screen_info->gnome_win);
+    grabKey (dpy, &screen_info->params->keys[KEY_UP_WORKSPACE], screen_info->gnome_win);
+    grabKey (dpy, &screen_info->params->keys[KEY_DOWN_WORKSPACE], screen_info->gnome_win);
+    grabKey (dpy, &screen_info->params->keys[KEY_LEFT_WORKSPACE], screen_info->gnome_win);
+    grabKey (dpy, &screen_info->params->keys[KEY_RIGHT_WORKSPACE], screen_info->gnome_win);
+
 
     return TRUE;
 }
@@ -980,12 +1047,17 @@ loadSettings (ScreenInfo *screen_info)
         {"margin_right", NULL, FALSE},
         {"margin_bottom", NULL, FALSE},
         {"margin_top", NULL, FALSE},
+        {"prevent_focus_stealing", NULL, TRUE},
         {"raise_delay", NULL, TRUE},
         {"raise_on_click", NULL, TRUE},
         {"raise_on_focus", NULL, TRUE},
         {"snap_to_border", NULL, TRUE},
         {"snap_to_windows", NULL, TRUE},
         {"snap_width", NULL, TRUE},
+        {"shadow_delta_x", NULL, TRUE},
+        {"shadow_delta_y", NULL, TRUE},
+        {"shadow_delta_width", NULL, TRUE},
+        {"shadow_delta_height", NULL, TRUE},
         {"theme", NULL, TRUE},
         {"title_alignment", NULL, TRUE},
         {"title_font", NULL, FALSE},
@@ -997,6 +1069,9 @@ loadSettings (ScreenInfo *screen_info)
         {"workspace_count", NULL, TRUE},
         {"wrap_windows", NULL, TRUE},
         {"wrap_workspaces", NULL, TRUE},
+        {"wrap_layout", NULL, TRUE},
+        {"wrap_cycle", NULL, TRUE},
+        {"scroll_workspaces", NULL, TRUE},
         {"wrap_resistance", NULL, TRUE},
         /* Keys */
         {"add_workspace_key", NULL, TRUE},
@@ -1062,6 +1137,14 @@ loadSettings (ScreenInfo *screen_info)
         {"shortcut_10_exec", NULL, FALSE},
         {"raise_window_key", NULL, TRUE},
         {"lower_window_key", NULL, TRUE},
+        {"up_workspace_key", NULL, TRUE},
+        {"down_workspace_key", NULL, TRUE},
+        {"left_workspace_key", NULL, TRUE},
+        {"right_workspace_key", NULL, TRUE},
+        {"move_window_up_workspace_key", NULL, TRUE},
+        {"move_window_down_workspace_key", NULL, TRUE},
+        {"move_window_left_workspace_key", NULL, TRUE},
+        {"move_window_right_workspace_key", NULL, TRUE},
         {NULL, NULL, FALSE}
     };
 
@@ -1095,6 +1178,8 @@ loadSettings (ScreenInfo *screen_info)
         !g_ascii_strcasecmp ("true", getValue ("focus_new", rc));
     screen_info->params->raise_on_focus =
         !g_ascii_strcasecmp ("true", getValue ("raise_on_focus", rc));
+    screen_info->params->prevent_focus_stealing =
+        !g_ascii_strcasecmp ("true", getValue ("prevent_focus_stealing", rc));
     screen_info->params->raise_delay = abs (TOINT (getValue ("raise_delay", rc)));
     screen_info->params->raise_on_click =
         !g_ascii_strcasecmp ("true", getValue ("raise_on_click", rc));
@@ -1105,7 +1190,6 @@ loadSettings (ScreenInfo *screen_info)
         !g_ascii_strcasecmp ("true", getValue ("snap_to_windows", rc));
     screen_info->params->snap_width = abs (TOINT (getValue ("snap_width", rc)));
 
-
     set_settings_margin (screen_info, LEFT,   TOINT (getValue ("margin_left", rc)));
     set_settings_margin (screen_info, RIGHT,  TOINT (getValue ("margin_right", rc)));
     set_settings_margin (screen_info, BOTTOM, TOINT (getValue ("margin_bottom", rc)));
@@ -1141,8 +1225,14 @@ loadSettings (ScreenInfo *screen_info)
 
     screen_info->params->wrap_workspaces =
         !g_ascii_strcasecmp ("true", getValue ("wrap_workspaces", rc));
+    screen_info->params->wrap_layout =
+        !g_ascii_strcasecmp ("true", getValue ("wrap_layout", rc));
     screen_info->params->wrap_windows =
         !g_ascii_strcasecmp ("true", getValue ("wrap_windows", rc));
+    screen_info->params->wrap_cycle =
+        !g_ascii_strcasecmp ("true", getValue ("wrap_cycle", rc));
+    screen_info->params->scroll_workspaces =
+        !g_ascii_strcasecmp ("true", getValue ("scroll_workspaces", rc));
     screen_info->params->wrap_resistance = abs (TOINT (getValue ("wrap_resistance", rc)));
 
     freeRc (rc);
@@ -1263,25 +1353,32 @@ initSettings (ScreenInfo *screen_info)
     {
         screen_info->params->shortcut_exec[i] = NULL;
     }
+
     if (!mcs_client_check_manager (myScreenGetXDisplay (screen_info), screen_info->screen, "xfce-mcs-manager"))
     {
         g_warning ("MCS manager not running, startup delayed for 5 seconds");
         sleep (5);
     }
+
     screen_info->mcs_client = mcs_client_new (myScreenGetXDisplay (screen_info), screen_info->screen, notify_cb, watch_cb, (gpointer) screen_info);
     if (screen_info->mcs_client)
     {
         mcs_client_add_channel (screen_info->mcs_client, CHANNEL1);
         mcs_client_add_channel (screen_info->mcs_client, CHANNEL2);
         mcs_client_add_channel (screen_info->mcs_client, CHANNEL3);
+        mcs_client_add_channel (screen_info->mcs_client, CHANNEL4);
+        mcs_client_set_raw_channel (screen_info->mcs_client, CHANNEL4, TRUE);
     }
     else
     {
         g_warning ("Cannot create MCS client channel");
     }
-
     screen_info->mcs_initted = TRUE;
 
+    if (!loadSettings (screen_info))
+    {
+        return FALSE;
+    }
     if (getHint (myScreenGetXDisplay (screen_info), screen_info->xroot, net_number_of_desktops, &val))
     {
         workspaceSetCount (screen_info, val);
@@ -1301,10 +1398,7 @@ initSettings (ScreenInfo *screen_info)
         screen_info->workspace_names_length = 0;
     }
 
-    if (!loadSettings (screen_info))
-    {
-        return FALSE;
-    }
+    getDesktopLayout(myScreenGetXDisplay (screen_info), screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
     placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
 
     return TRUE;
